// Music 256a / CS 476a | fall 2016
// CCRMA, Stanford University
//
// Author: Romain Michon (rmichonATccrmaDOTstanfordDOTedu)
// Description: Faust-based stereo reverb effect.

#ifndef MAINCOMPONENT_H_INCLUDED
#define MAINCOMPONENT_H_INCLUDED

#include "../JuceLibraryCode/JuceHeader.h"
#include "faust/FaustReverb.h" // the Faust module

//==============================================================================
/*
    This component lives inside our window, and this is where you should put all
    your controls and content.
*/
class MainContentComponent :
    public AudioAppComponent,
    private Slider::Listener
{
public:
    //==============================================================================
    MainContentComponent() : currentSampleRate(0.0)
    {
        addAndMakeVisible (wetSlider);
        wetSlider.setRange (-1.0, 1.0);
        wetSlider.setValue(0.0);
        wetSlider.addListener (this);
        
        addAndMakeVisible(wetLabel);
        wetLabel.setText ("Wet/Dry", dontSendNotification);
        wetLabel.attachToComponent (&wetSlider, true);
        
        addAndMakeVisible (levelSlider);
        levelSlider.setRange (-70.0, 40.0);
        levelSlider.setValue(-20.0);
        levelSlider.addListener (this);
        
        addAndMakeVisible(levelLabel);
        levelLabel.setText ("Level", dontSendNotification);
        levelLabel.attachToComponent (&levelSlider, true);
        
        setSize (600, 70);

        nChans = 2; // number of input and output audio channels
        setAudioChannels (nChans, nChans);
        
        audioBuffer = new float*[nChans];
    }

    ~MainContentComponent()
    {
        shutdownAudio();
        delete [] audioBuffer;
    }

    //==============================================================================
    void prepareToPlay (int samplesPerBlockExpected, double sampleRate) override
    {
        currentSampleRate = sampleRate;
        blockSize = samplesPerBlockExpected;
        
        reverb.init(sampleRate); // initializing the Faust module
        reverb.buildUserInterface(&reverbControl); // linking the Faust module to the controler
        
        // Print the list of parameters address of "reverb"
        // To get the current (default) value of these parameters, reverbControl.getParamValue("paramPath") can be used
        for(int i=0; i<reverbControl.getParamsCount(); i++){
            std::cout << reverbControl.getParamAdress(i) << "\n";
        }
        
        // setting default values for the Faust module parameters
        reverbControl.setParamValue("/Zita_Rev1/Output/Level",-20);
        reverbControl.setParamValue("/Zita_Rev1/Output/Dry/Wet_Mix",0.0);
    }
    
    void getNextAudioBlock (const AudioSourceChannelInfo& bufferToFill) override
    {
        for(int i=0; i<nChans; i++){
            audioBuffer[i] = bufferToFill.buffer->getWritePointer (i, bufferToFill.startSample);
        }
        // any processing before going to "reverb" could be done here in a dedicated buffer loop
        
        reverb.compute(blockSize, audioBuffer, audioBuffer); // computing one block with Faust
        
        // any processing after going to "reverb" could be done here in a dedicated buffer loop
        // An alternative solution could be to use only one buffer loop and to set the blockSize
        // of "reverb.compute" to 1...
    }
    
    void releaseResources() override
    {
        // This will be called when the audio device stops, or when it is being
        // restarted due to a setting change.

        // For more details, see the help for AudioProcessor::releaseResources()
    }

    void resized() override
    {
        const int sliderLeft = 80;
        wetSlider.setBounds (sliderLeft, 10, getWidth() - sliderLeft - 20, 20);
        levelSlider.setBounds (sliderLeft, 40, getWidth() - sliderLeft - 20, 20);
    }
    
    void sliderValueChanged (Slider* slider) override
    {
        if (currentSampleRate > 0.0){
            if (slider == &wetSlider)
            {
                reverbControl.setParamValue("/Zita_Rev1/Output/Dry/Wet_Mix", wetSlider.getValue());
            }
            else if (slider == &levelSlider)
            {
                reverbControl.setParamValue("/Zita_Rev1/Output/Level", levelSlider.getValue());
            }
        }
    }
    
private:
    FaustReverb reverb; // the Faust module (Reverb.h)
    MapUI reverbControl; // used to easily control the Faust module (Reverb.h)
    
    Slider wetSlider;
    Slider levelSlider;
    
    Label wetLabel;
    Label levelLabel;
    
    int blockSize, nChans;
    double currentSampleRate;
    
    float** audioBuffer; // multichannel audio buffer used both for input and output

    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainContentComponent)
};


// (This function is called by the app startup code to create our main component)
Component* createMainContentComponent()     { return new MainContentComponent(); }


#endif  // MAINCOMPONENT_H_INCLUDED
